import os
import re
from fastapi import APIRouter
import importlib
import inspect

METHODS = {
    'create': {
        'method': 'POST',
    },
    'find_all': {
        'method': 'GET',
    },
    'find_one': {
        'method': 'GET',
        'pk': True
    },
    'update_one': {
        'method': 'PATCH',
        'pk': True
    },
    'update_all': {
        'method': 'PUT',
        'pk': True
    },
    'remove': {
        'method': 'DELETE',
        'pk': True
    }
}


class DynamicRouter(APIRouter):
    def __init__(
        self,
        base: str = "apps",
        file: str = 'controller.py',
        *args,
        **kwargs,
    ):
        self._base = base
        self._file = file
        super().__init__(*args, **kwargs)
        self.build_router()

    def build_router(self):
        file_list = []
        for root, dirs, files in os.walk(os.path.join(os.getcwd(), self.base)):
            for file in files:
                if re.match(f'{self._file}$', str(file)):
                    _file = os.path.join(root, file)
                    _file_router = re.sub('.py', '', str(_file))
                    file_list.append(os.path.join(root, file))
                    route_file = importlib.import_module(
                        os.path.relpath(root).replace(
                            "/", ".").replace('\\', '.') + "." + re.sub('.py', '', str(file))
                    )
                    find_routes = [r for r in dir(route_file) if r in METHODS.keys()]
                    if find_routes:
                        if os.path.basename(root) == self.base:
                            tags = ["default"]
                        else:
                            tags = [re.sub(
                                "[\/]controller$", "", str(os.path.relpath(_file_router, self.base)).replace("\\", "/"))]

                        # Replace \\ in window
                        uri = str(os.path.relpath(_file_router,
                                  self.base)).replace("\\", "/")
                        uri = re.sub("[\/]controller$", "", uri)
                        _router = APIRouter(
                            prefix="/" + uri,
                            tags=tags,
                        )

                        for r in find_routes:
                            func = getattr(route_file, r)
                            arg = self.get_first_arg(func)
                            suffix = '/' if len(arg) == 0 else '/{' + arg[0] +'}/'
                            _router.add_api_route(suffix, getattr(route_file, r), methods=[METHODS.get(r).get('method')])

                        self.include_router(_router)

    def convert_to_lower_with_hyphen(filename):
        lower_router = filename[0].lower()
        for char in filename[1:]:
            if char.isupper():
                lower_router += '-' + char.lower()
            else:
                lower_router += char

        return lower_router

    def get_first_arg(self, func):
        signature = inspect.signature(func)
        params = [param.name for param in signature.parameters.values()]
        return list(filter(lambda x: x not in ['kwargs'], params))
    
    @property
    def base(self):
        return self._base

    @base.setter
    def base(self, value):
        self._base = value



# Copyright 2023 Hao Hoang (haohoangofficial@gmail.com)

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import subprocess

try:
    import fastapi
except ImportError:
    try:
        command = ["pip", "install", "fastapi"]
        subprocess.run(command, check=True)
    except Exception:
        command = ["pip3", "install", "fastapi"]
        subprocess.run(command, check=True)
import os

this_dir = os.path.dirname(__file__)
readme_filename = os.path.join(this_dir, 'README.md')
requirements_filename = os.path.join(this_dir, 'requirements.txt')

PACKAGE_NAME = 'phowr-router'
PACKAGE_VERSION = '1.0.2'
PACKAGE_AUTHOR = 'Hao Hoang'
PACKAGE_AUTHOR_EMAIL = 'haohoangofficial@gmail.com'
PACKAGE_URL = 'https://github.com/hdsme/phowr-router'
PACKAGE_DOWNLOAD_URL = \
    'https://github.com/hdsme/phowr-router/tarball/' + PACKAGE_VERSION
PACKAGES = [
    'phowr_router',
    'phowr_router/core',
]
PACKAGE_DATA = {}
PACKAGE_LICENSE = 'LICENSE.txt'
PACKAGE_DESCRIPTION = 'Phowr Router'

with open(readme_filename) as f:
    PACKAGE_LONG_DESCRIPTION = f.read()

with open(requirements_filename) as f:
    PACKAGE_INSTALL_REQUIRES = [line[:-1] for line in f]

setup(
    name=PACKAGE_NAME,
    version=PACKAGE_VERSION,
    author=PACKAGE_AUTHOR,
    author_email=PACKAGE_AUTHOR_EMAIL,
    url=PACKAGE_URL,
    download_url=PACKAGE_DOWNLOAD_URL,
    packages=PACKAGES,
    package_data=PACKAGE_DATA,
    license=PACKAGE_LICENSE,
    description=PACKAGE_DESCRIPTION,
    long_description=PACKAGE_LONG_DESCRIPTION,
    install_requires=PACKAGE_INSTALL_REQUIRES,
    long_description_content_type="text/markdown",
)